@@ -345,7 +345,7 @@ class ElectrumWindow(QMainWindow):
         wallet = wizard.run()
         if wallet: 
             self.load_wallet(wallet)
-        
+
 
 
     def init_menubar(self):
@@ -672,6 +672,9 @@ class ElectrumWindow(QMainWindow):
     def current_item_changed(self, a):
         run_hook('current_item_changed', a)
 
+        self.pay_from = []
+        self.tabs.emit(SIGNAL('currentChanged(int)'), 1)
+
 
 
     def update_history_tab(self):
@@ -754,21 +757,33 @@ class ElectrumWindow(QMainWindow):
         grid.addWidget(self.message_e, 2, 1, 1, 3)
         grid.addWidget(HelpButton(_('Description of the transaction (not mandatory).') + '\n\n' + _('The description is not sent to the recipient of the funds. It is stored in your wallet file, and displayed in the \'History\' tab.')), 2, 4)
 
+        self.pay_from = []
+        self.from_label = QLabel(_('From'))
+        grid.addWidget(self.from_label, 3, 0)
+        self.from_list = QTreeWidget(self)
+        self.from_list.setColumnCount(2)
+        self.from_list.setColumnWidth(0, 350)
+        self.from_list.setColumnWidth(1, 50)
+        self.from_list.setHeaderHidden (True)
+        self.from_list.setMaximumHeight(80)
+        grid.addWidget(self.from_list, 3, 1, 1, 3)
+        self.connect(self.tabs, SIGNAL('currentChanged(int)'), lambda: self.update_pay_from_list(grid))
+
         self.amount_e = AmountEdit(self.base_unit)
-        grid.addWidget(QLabel(_('Amount')), 3, 0)
-        grid.addWidget(self.amount_e, 3, 1, 1, 2)
+        grid.addWidget(QLabel(_('Amount')), 4, 0)
+        grid.addWidget(self.amount_e, 4, 1, 1, 2)
         grid.addWidget(HelpButton(
                 _('Amount to be sent.') + '\n\n' \
                     + _('The amount will be displayed in red if you do not have enough funds in your wallet. Note that if you have frozen some of your addresses, the available funds will be lower than your total balance.') \
-                    + '\n\n' + _('Keyboard shortcut: type "!" to send all your coins.')), 3, 3)
+                    + '\n\n' + _('Keyboard shortcut: type "!" to send all your coins.')), 4, 3)
         
         self.fee_e = AmountEdit(self.base_unit)
-        grid.addWidget(QLabel(_('Fee')), 4, 0)
-        grid.addWidget(self.fee_e, 4, 1, 1, 2) 
+        grid.addWidget(QLabel(_('Fee')), 5, 0)
+        grid.addWidget(self.fee_e, 5, 1, 1, 2)
         grid.addWidget(HelpButton(
                 _('Bitcoin transactions are in general not free. A transaction fee is paid by the sender of the funds.') + '\n\n'\
                     + _('The amount of fee can be decided freely by the sender. However, transactions with low fees take more time to be processed.') + '\n\n'\
-                    + _('A suggested fee is automatically added to this field. You may override it. The suggested fee increases with the size of the transaction.')), 4, 3)
+                    + _('A suggested fee is automatically added to this field. You may override it. The suggested fee increases with the size of the transaction.')), 5, 3)
 
 
         self.send_button = EnterButton(_("Send"), self.do_send)
@@ -795,8 +810,8 @@ class ElectrumWindow(QMainWindow):
 
             if self.amount_e.is_shortcut:
                 self.amount_e.is_shortcut = False
-                c, u = self.wallet.get_account_balance(self.current_account)
-                inputs, total, fee = self.wallet.choose_tx_inputs_from_account( c + u, 0, self.current_account)
+                sendable = self.get_sendable_balance()
+                inputs, total, fee = self.wallet.choose_tx_inputs( sendable, 0, self.get_payment_sources())
                 fee = self.wallet.estimated_fee(inputs)
                 amount = total - fee
                 self.amount_e.setText( self.format_amount(amount) )
@@ -809,7 +824,7 @@ class ElectrumWindow(QMainWindow):
             if not is_fee: fee = None
             if amount is None:
                 return
-            inputs, total, fee = self.wallet.choose_tx_inputs_from_account( amount, fee, self.current_account )
+            inputs, total, fee = self.wallet.choose_tx_inputs(amount, fee, self.get_payment_sources())
             if not is_fee:
                 self.fee_e.setText( self.format_amount( fee ) )
             if inputs:
@@ -835,6 +850,16 @@ class ElectrumWindow(QMainWindow):
         return w2
 
 
+    def update_pay_from_list(self, grid):
+        self.from_list.clear()
+        self.from_label.setHidden(len(self.pay_from) == 0)
+        self.from_list.setHidden(len(self.pay_from) == 0)
+        for addr in self.pay_from:
+            c, u = self.wallet.get_addr_balance(addr)
+            balance = self.format_amount(c + u)
+            self.from_list.addTopLevelItem(QTreeWidgetItem( [addr, balance] ))
+
+
     def update_completions(self):
         l = []
         for addr,label in self.wallet.labels.items():
@@ -884,9 +909,9 @@ class ElectrumWindow(QMainWindow):
 
     @protected
     def send_tx(self, to_address, amount, fee, label, password):
-
         try:
-            tx = self.wallet.mktx_from_account( [(to_address, amount)], password, fee, self.current_account)
+            tx = self.wallet.mktx( [(to_address, amount)], password, fee,
+                    domain=self.get_payment_sources())
         except Exception as e:
             traceback.print_exc(file=sys.stdout)
             self.show_message(str(e))
@@ -910,14 +935,8 @@ class ElectrumWindow(QMainWindow):
             else:
                 QMessageBox.warning(self, _('Error'), msg, _('OK'))
         else:
-            filename = label + '.txn' if label else 'unsigned_%s.txn' % (time.mktime(time.gmtime()))
-            try:
-                fileName = self.getSaveFileName(_("Select a transaction filename"), filename, "*.txn")
-                with open(fileName,'w') as f:
-                    f.write(json.dumps(tx.as_dict(),indent=4) + '\n')
-                QMessageBox.information(self, _('Unsigned transaction created'), _("Unsigned transaction was saved to file:") + " " +fileName, _('OK'))
-            except Exception:
-                QMessageBox.warning(self, _('Error'), _('Could not write transaction to file'), _('OK'))
+
+            self.show_transaction(tx)
 
         # add recipient to addressbook
         if to_address not in self.wallet.addressbook and not self.wallet.is_mine(to_address):
@@ -929,7 +948,12 @@ class ElectrumWindow(QMainWindow):
     def set_url(self, url):
         address, amount, label, message, signature, identity, url = util.parse_url(url)
 
-        if amount and self.base_unit() == 'mBTC': amount = str( 1000* Decimal(amount))
+        try:
+            if amount and self.base_unit() == 'mBTC': amount = str( 1000* Decimal(amount))
+            elif amount: amount = str(Decimal(amount))
+        except Exception:
+            amount = "0.0"
+            QMessageBox.warning(self, _('Error'), _('Invalid Amount'), _('OK'))
 
         if self.mini:
             self.mini.set_payment_fields(address, amount)
@@ -964,6 +988,10 @@ class ElectrumWindow(QMainWindow):
         for e in [self.payto_e, self.message_e, self.amount_e, self.fee_e]:
             e.setText('')
             self.set_frozen(e,False)
+
+        self.pay_from = []
+        self.tabs.emit(SIGNAL('currentChanged(int)'), 1)
+
         self.update_status()
 
     def set_frozen(self,entry,frozen):
@@ -981,21 +1009,15 @@ class ElectrumWindow(QMainWindow):
             entry.setPalette(palette)
 
 
-    def toggle_freeze(self,addr):
-        if not addr: return
-        if addr in self.wallet.frozen_addresses:
-            self.wallet.unfreeze(addr)
-        else:
-            self.wallet.freeze(addr)
+    def set_addrs_frozen(self,addrs,freeze):
+        for addr in addrs:
+            if not addr: continue
+            if addr in self.wallet.frozen_addresses and not freeze:
+                self.wallet.unfreeze(addr)
+            elif addr not in self.wallet.frozen_addresses and freeze:
+                self.wallet.freeze(addr)
         self.update_receive_tab()
 
-    def toggle_priority(self,addr):
-        if not addr: return
-        if addr in self.wallet.prioritized_addresses:
-            self.wallet.unprioritize(addr)
-        else:
-            self.wallet.prioritize(addr)
-        self.update_receive_tab()
 
 
     def create_list_tab(self, headers):
@@ -1026,6 +1048,7 @@ class ElectrumWindow(QMainWindow):
         l,w,hbox = self.create_list_tab([ _('Address'), _('Label'), _('Balance'), _('Tx')])
         l.setContextMenuPolicy(Qt.CustomContextMenu)
         l.customContextMenuRequested.connect(self.create_receive_menu)
+        l.setSelectionMode(QAbstractItemView.ExtendedSelection)
         self.connect(l, SIGNAL('itemDoubleClicked(QTreeWidgetItem*, int)'), lambda a, b: self.address_label_clicked(a,b,l,0,1))
         self.connect(l, SIGNAL('itemChanged(QTreeWidgetItem*, int)'), lambda a,b: self.address_label_changed(a,b,l,0,1))
         self.connect(l, SIGNAL('currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)'), lambda a,b: self.current_item_changed(a))
@@ -1106,37 +1129,60 @@ class ElectrumWindow(QMainWindow):
         # if it is not called the menu pops up several times
         #self.receive_list.selectedIndexes() 
 
-        item = self.receive_list.itemAt(position)
-        if not item: return
+        selected = self.receive_list.selectedItems()
+        multi_select = len(selected) > 1
+        addrs = [unicode(item.text(0)) for item in selected]
+        if not multi_select:
+            item = self.receive_list.itemAt(position)
+            if not item: return
 
-        addr = unicode(item.text(0))
-        if not is_valid(addr): 
-            k = str(item.data(0,32).toString())
-            if k:
-                self.create_account_menu(position, k, item)
-            else:
-                item.setExpanded(not item.isExpanded())
-            return 
+            addr = addrs[0]
+            if not is_valid(addr):
+                k = str(item.data(0,32).toString())
+                if k:
+                    self.create_account_menu(position, k, item)
+                else:
+                    item.setExpanded(not item.isExpanded())
+                return
 
         menu = QMenu()
-        menu.addAction(_("Copy to clipboard"), lambda: self.app.clipboard().setText(addr))
-        menu.addAction(_("QR code"), lambda: self.show_qrcode("bitcoin:" + addr, _("Address")) )
-        menu.addAction(_("Edit label"), lambda: self.edit_label(True))
-        if self.wallet.seed:
-            menu.addAction(_("Private key"), lambda: self.show_private_key(addr))
-            menu.addAction(_("Sign message"), lambda: self.sign_message(addr))
-        if addr in self.wallet.imported_keys:
-            menu.addAction(_("Remove from wallet"), lambda: self.delete_imported_key(addr))
+        if not multi_select:
+            menu.addAction(_("Copy to clipboard"), lambda: self.app.clipboard().setText(addr))
+            menu.addAction(_("QR code"), lambda: self.show_qrcode("bitcoin:" + addr, _("Address")) )
+            menu.addAction(_("Edit label"), lambda: self.edit_label(True))
+            if self.wallet.seed:
+                menu.addAction(_("Private key"), lambda: self.show_private_key(addr))
+                menu.addAction(_("Sign message"), lambda: self.sign_message(addr))
+            if addr in self.wallet.imported_keys:
+                menu.addAction(_("Remove from wallet"), lambda: self.delete_imported_key(addr))
 
-        t = _("Unfreeze") if addr in self.wallet.frozen_addresses else _("Freeze")
-        menu.addAction(t, lambda: self.toggle_freeze(addr))
-        t = _("Unprioritize") if addr in self.wallet.prioritized_addresses else _("Prioritize")
-        menu.addAction(t, lambda: self.toggle_priority(addr))
+        if any(addr not in self.wallet.frozen_addresses for addr in addrs):
+            menu.addAction(_("Freeze"), lambda: self.set_addrs_frozen(addrs, True))
+        if any(addr in self.wallet.frozen_addresses for addr in addrs):
+            menu.addAction(_("Unfreeze"), lambda: self.set_addrs_frozen(addrs, False))
+
+        menu.addAction(_("Send From"), lambda: self.send_from_addresses(addrs))
             
         run_hook('receive_menu', menu)
         menu.exec_(self.receive_list.viewport().mapToGlobal(position))
 
 
+    def get_sendable_balance(self):
+        return sum(sum(self.wallet.get_addr_balance(a)) for a in self.get_payment_sources())
+
+
+    def get_payment_sources(self):
+        if self.pay_from:
+            return self.pay_from
+        else:
+            return self.wallet.get_account_addresses(self.current_account)
+
+
+    def send_from_addresses(self, addrs):
+        self.pay_from = addrs[:]
+        self.tabs.setCurrentIndex(1)
+
+
     def payto(self, addr):
         if not addr: return
         label = self.wallet.labels.get(addr)
@@ -1191,9 +1237,7 @@ class ElectrumWindow(QMainWindow):
 
         if address in self.wallet.frozen_addresses: 
             item.setBackgroundColor(0, QColor('lightblue'))
-        elif address in self.wallet.prioritized_addresses: 
-            item.setBackgroundColor(0, QColor('lightgreen'))
-        
+
 
     def update_receive_tab(self):
         l = self.receive_list
@@ -1227,6 +1271,8 @@ class ElectrumWindow(QMainWindow):
                 name = _("Receiving") if not is_change else _("Change")
                 seq_item = QTreeWidgetItem( [ name, '', '', '', ''] )
                 account_item.addChild(seq_item)
+                used_item = QTreeWidgetItem( [ _("Used"), '', '', '', ''] )
+                used_flag = False
                 if not is_change: seq_item.setExpanded(True)
 
                 is_red = False
@@ -1242,12 +1288,19 @@ class ElectrumWindow(QMainWindow):
                     else:
                         gap = 0
 
+                    c, u = self.wallet.get_addr_balance(address)
                     num_tx = '*' if h == ['*'] else "%d"%len(h)
                     item = QTreeWidgetItem( [ address, '', '', num_tx] )
                     self.update_receive_item(item)
                     if is_red:
                         item.setBackgroundColor(1, QColor('red'))
-                    seq_item.addChild(item)
+                    if len(h) > 0 and c == -u:
+                        if not used_flag:
+                            seq_item.addChild(used_item)
+                            used_flag = True
+                        used_item.addChild(item)
+                    else:
+                        seq_item.addChild(item)
 
 
         for k, addr in self.wallet.get_pending_accounts():
@@ -1999,7 +2052,7 @@ class ElectrumWindow(QMainWindow):
         fee_e.setText(self.format_amount(self.wallet.fee).strip())
         grid.addWidget(fee_e, 2, 1)
         msg = _('Fee per kilobyte of transaction.') + ' ' \
-            + _('Recommended value') + ': ' + self.format_amount(50000)
+            + _('Recommended value') + ': ' + self.format_amount(20000)
         grid.addWidget(HelpButton(msg), 2, 2)
         if not self.config.is_modifiable('fee_per_kb'):
             for w in [fee_e, fee_label]: w.setEnabled(False)
