@@ -125,7 +125,10 @@ class WalletStorage:
 
 
     def get(self, key, default=None):
-        return self.data.get(key, default)
+        v = self.data.get(key)
+        if v is None: 
+            v = default
+        return v
 
     def put(self, key, value, save = True):
 
@@ -164,7 +167,6 @@ class Wallet:
         self.seed                  = storage.get('seed', '')               # encrypted
         self.labels                = storage.get('labels', {})
         self.frozen_addresses      = storage.get('frozen_addresses',[])
-        self.prioritized_addresses = storage.get('prioritized_addresses',[])
         self.addressbook           = storage.get('contacts', [])
 
         self.imported_keys         = storage.get('imported_keys',{})
@@ -984,7 +986,7 @@ class Wallet:
 
     def get_address_flags(self, addr):
         flags = "C" if self.is_change(addr) else "I" if addr in self.imported_keys.keys() else "-" 
-        flags += "F" if addr in self.frozen_addresses else "P" if addr in self.prioritized_addresses else "-"
+        flags += "F" if addr in self.frozen_addresses else "-"
         return flags
         
 
@@ -1138,12 +1140,6 @@ class Wallet:
         return [x[1] for x in coins]
 
 
-
-    def choose_tx_inputs_from_account( self, amount, fixed_fee, account ):
-        domain = self.get_account_addresses(account) if account else None
-        return self.choose_tx_inputs( amount, fixed_fee, domain )
-
-
     def choose_tx_inputs( self, amount, fixed_fee, domain = None ):
         """ todo: minimize tx size """
         total = 0
@@ -1154,17 +1150,8 @@ class Wallet:
         for i in self.frozen_addresses:
             if i in domain: domain.remove(i)
 
-        prioritized = []
-        for i in self.prioritized_addresses:
-            if i in domain:
-                domain.remove(i)
-                prioritized.append(i)
-
         coins = self.get_unspent_coins(domain)
-        prioritized_coins = self.get_unspent_coins(prioritized)
-
         inputs = []
-        coins = prioritized_coins + coins
 
         for item in coins:
             if item.get('coinbase') and item.get('height') + COINBASE_MATURITY > self.network.blockchain.height:
@@ -1357,11 +1344,6 @@ class Wallet:
         return Transaction.from_io(inputs, outputs)
 
 
-    def mktx_from_account(self, outputs, password, fee=None, account=None):
-        domain = self.get_account_addresses(account) if account else None
-        return self.mktx(outputs, password, fee, change_addr=None, domain=domain)
-
-
     def mktx(self, outputs, password, fee=None, change_addr=None, domain= None ):
         tx = self.make_unsigned_transaction(outputs, fee, change_addr, domain)
         keypairs = {}
@@ -1437,7 +1419,6 @@ class Wallet:
 
     def freeze(self,addr):
         if self.is_mine(addr) and addr not in self.frozen_addresses:
-            self.unprioritize(addr)
             self.frozen_addresses.append(addr)
             self.storage.put('frozen_addresses', self.frozen_addresses, True)
             return True
@@ -1452,23 +1433,6 @@ class Wallet:
         else:
             return False
 
-    def prioritize(self,addr):
-        if self.is_mine(addr) and addr not in self.prioritized_addresses:
-            self.unfreeze(addr)
-            self.prioritized_addresses.append(addr)
-            self.storage.put('prioritized_addresses', self.prioritized_addresses, True)
-            return True
-        else:
-            return False
-
-    def unprioritize(self,addr):
-        if self.is_mine(addr) and addr in self.prioritized_addresses:
-            self.prioritized_addresses.remove(addr)
-            self.storage.put('prioritized_addresses', self.prioritized_addresses, True)
-            return True
-        else:
-            return False
-
 
     def set_verifier(self, verifier):
         self.verifier = verifier
